@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: cryptokit.ml 71 2013-04-23 17:41:40Z xleroy $ *)
+(* $Id: cryptokit.ml 88 2014-12-27 09:02:04Z xleroy $ *)
 
 open Nat
 
@@ -60,8 +60,15 @@ external sha1_init: unit -> string = "caml_sha1_init"
 external sha1_update: string -> string -> int -> int -> unit = "caml_sha1_update"
 external sha1_final: string -> string = "caml_sha1_final"
 external sha256_init: unit -> string = "caml_sha256_init"
+external sha224_init: unit -> string = "caml_sha224_init"
 external sha256_update: string -> string -> int -> int -> unit = "caml_sha256_update"
 external sha256_final: string -> string = "caml_sha256_final"
+external sha224_final: string -> string = "caml_sha224_final"
+external sha512_init: unit -> string = "caml_sha512_init"
+external sha384_init: unit -> string = "caml_sha384_init"
+external sha512_update: string -> string -> int -> int -> unit = "caml_sha512_update"
+external sha512_final: string -> string = "caml_sha512_final"
+external sha384_final: string -> string = "caml_sha384_final"
 type sha3_context
 external sha3_init: int -> sha3_context = "caml_sha3_init"
 external sha3_absorb: sha3_context -> string -> int -> int -> unit = "caml_sha3_absorb"
@@ -558,6 +565,34 @@ class ofb ?iv:iv_init chunksize (cipher : block_cipher) =
       wipe_string iv
   end
 
+let rec increment_counter c lim pos =
+  if pos >= lim then begin
+    let i = 1 + Char.code c.[pos] in
+    c.[pos] <- Char.unsafe_chr i;
+    if i = 0x100 then increment_counter c lim (pos - 1)
+  end
+
+class ctr ?iv:iv_init ?inc (cipher : block_cipher) =
+  let blocksize = cipher#blocksize in
+  let nincr =
+    match inc with
+    | None -> blocksize
+    | Some n -> assert (n > 0 && n <= blocksize); n in
+  object(self)
+    val iv = make_initial_iv blocksize iv_init
+    val out = String.create blocksize
+    method blocksize = blocksize
+    method transform src src_off dst dst_off =
+      cipher#transform iv 0 out 0;
+      String.blit src src_off dst dst_off blocksize;
+      xor_string out 0 dst dst_off blocksize;
+      increment_counter iv (blocksize - nincr) (blocksize - 1)
+    method wipe =
+      cipher#wipe;
+      wipe_string iv;
+      wipe_string out
+  end
+
 (* Wrapping of a block cipher as a transform *)
 
 class cipher (cipher : block_cipher) =
@@ -842,6 +877,28 @@ class sha1 =
 
 let sha1 () = new sha1
 
+class sha224 =
+  object(self)
+    val context = sha224_init()
+    method hash_size = 24
+    method add_substring src ofs len =
+      if ofs < 0 || ofs + len > String.length src
+      then invalid_arg "sha224#add_substring";
+      sha256_update context src ofs len
+    method add_string src =
+      sha256_update context src 0 (String.length src)
+    method add_char c =
+      self#add_string (String.make 1 c)
+    method add_byte b =
+      self#add_char (Char.unsafe_chr b)
+    method result =
+      sha224_final context
+    method wipe =
+      wipe_string context
+  end
+
+let sha224 () = new sha224
+
 class sha256 =
   object(self)
     val context = sha256_init()
@@ -864,6 +921,58 @@ class sha256 =
 
 let sha256 () = new sha256
 
+class sha384 =
+  object(self)
+    val context = sha384_init()
+    method hash_size = 48
+    method add_substring src ofs len =
+      if ofs < 0 || ofs + len > String.length src
+      then invalid_arg "sha384#add_substring";
+      sha512_update context src ofs len
+    method add_string src =
+      sha512_update context src 0 (String.length src)
+    method add_char c =
+      self#add_string (String.make 1 c)
+    method add_byte b =
+      self#add_char (Char.unsafe_chr b)
+    method result =
+      sha384_final context
+    method wipe =
+      wipe_string context
+  end
+
+let sha384 () = new sha384
+
+class sha512 =
+  object(self)
+    val context = sha512_init()
+    method hash_size = 64
+    method add_substring src ofs len =
+      if ofs < 0 || ofs + len > String.length src
+      then invalid_arg "sha512#add_substring";
+      sha512_update context src ofs len
+    method add_string src =
+      sha512_update context src 0 (String.length src)
+    method add_char c =
+      self#add_string (String.make 1 c)
+    method add_byte b =
+      self#add_char (Char.unsafe_chr b)
+    method result =
+      sha512_final context
+    method wipe =
+      wipe_string context
+  end
+
+let sha512 () = new sha512
+
+let sha2 sz =
+  match sz with
+  | 224 -> new sha224
+  | 256 -> new sha256
+  | 384 -> new sha384
+  | 512 -> new sha512
+  |  _  -> raise (Error Wrong_key_size)
+
 class sha3 sz =
   object(self)
     val context =
@@ -946,6 +1055,8 @@ type chaining_mode =
   | CBC
   | CFB of int
   | OFB of int
+  | CTR
+  | CTR_N of int
 
 let make_block_cipher ?(mode = CBC) ?pad ?iv dir block_cipher =
   let chained_cipher =
@@ -955,7 +1066,9 @@ let make_block_cipher ?(mode = CBC) ?pad ?iv dir block_cipher =
     | (CBC, Decrypt) -> new Block.cbc_decrypt ?iv block_cipher
     | (CFB n, Encrypt) -> new Block.cfb_encrypt ?iv n block_cipher
     | (CFB n, Decrypt) -> new Block.cfb_decrypt ?iv n block_cipher
-    | (OFB n, _) -> new Block.ofb ?iv n block_cipher in
+    | (OFB n, _) -> new Block.ofb ?iv n block_cipher
+    | (CTR, _) -> new Block.ctr ?iv block_cipher
+    | (CTR_N n, _) -> new Block.ctr ?iv ~inc:n block_cipher in
   match pad with
     None -> new Block.cipher chained_cipher
   | Some p ->
@@ -965,7 +1078,7 @@ let make_block_cipher ?(mode = CBC) ?pad ?iv dir block_cipher =
 
 let normalize_dir mode dir =
   match mode with
-    Some(CFB _) | Some(OFB _) -> Encrypt
+  | Some(CFB _) | Some(OFB _) | Some(CTR) | Some(CTR_N _) -> Encrypt
   | _ -> dir
 
 let aes ?mode ?pad ?iv key dir =
@@ -1033,6 +1146,8 @@ module HMAC_SHA1 =
   HMAC(struct class h = Hash.sha1  let blocksize = 64 end)
 module HMAC_SHA256 =
   HMAC(struct class h = Hash.sha256  let blocksize = 64 end)
+module HMAC_SHA512 =
+  HMAC(struct class h = Hash.sha512  let blocksize = 128 end)
 module HMAC_RIPEMD160 = 
   HMAC(struct class h = Hash.ripemd160  let blocksize = 64 end)
 module HMAC_MD5 =
@@ -1040,6 +1155,7 @@ module HMAC_MD5 =
 
 let hmac_sha1 key = new HMAC_SHA1.hmac key
 let hmac_sha256 key = new HMAC_SHA256.hmac key
+let hmac_sha512 key = new HMAC_SHA512.hmac key
 let hmac_ripemd160 key = new HMAC_RIPEMD160.hmac key
 let hmac_md5 key = new HMAC_MD5.hmac key
 
@@ -1927,7 +2043,7 @@ class encode multiline padding =
           oend <- oend + 3
       | _ -> ()
       end;
-      if multiline or padding then begin
+      if multiline || padding then begin
         let num_equals =
           match ipos with 1 -> 2 | 2 -> 1 | _ -> 0 in
         self#ensure_capacity num_equals;
@@ -1966,7 +2082,7 @@ class decode =
     method input_block_size = 1
     method output_block_size = 1
 
-    val ibuf = Array.create 4 0
+    val ibuf = Array.make 4 0
     val mutable ipos = 0
     val mutable finished = false
 
@@ -2071,7 +2187,7 @@ class decode =
     method input_block_size = 1
     method output_block_size = 1
 
-    val ibuf = Array.create 2 0
+    val ibuf = Array.make 2 0
     val mutable ipos = 0
 
     method put_char c =
