@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: cryptokit.mli 71 2013-04-23 17:41:40Z xleroy $ *)
+(* $Id: cryptokit.mli 88 2014-12-27 09:02:04Z xleroy $ *)
 
 (** The Cryptokit library provides a variety of cryptographic primitives
     that can be used to implement cryptographic protocols in
@@ -19,7 +19,7 @@
     - Symmetric-key ciphers: AES, DES, Triple-DES, ARCfour,
       in ECB, CBC, CFB and OFB modes.
     - Public-key cryptography: RSA encryption, Diffie-Hellman key agreement.
-    - Hash functions and MACs: SHA-1, SHA-256, SHA-3, RIPEMD-160, MD5,
+    - Hash functions and MACs: SHA-1, SHA-256, SHA-512. SHA-3, RIPEMD-160, MD5,
       and MACs based on AES and DES.
     - Random number generation.
     - Encodings and compression: base 64, hexadecimal, Zlib compression.
@@ -340,14 +340,21 @@ module Cipher : sig
     | CBC
     | CFB of int
     | OFB of int
+    | CTR
+    | CTR_N of int
     (** Block ciphers such as AES or DES map a fixed-sized block of
         input data to a block of output data of the same size.
         A chaining mode indicates how to extend them to multiple blocks
-        of data.  The four chaining modes supported in this library are:
+        of data.  The five chaining modes supported in this library are:
         - [ECB]: Electronic Code Book mode.
         - [CBC]: Cipher Block Chaining mode.
         - [CFB n]:  Cipher Feedback Block with [n] bytes.
-        - [OFB n]: Output Feedback Block with [n] bytes.
+        - [OFB n]: Output Feedback Block with [n] bytes
+        - [CTR]: Counter mode, incrementing all the bytes of the IV
+        - [CTR_N n]: Counter mode, incrementing only the final [n]
+          bytes of the IV.  For example, [CTR_N 4] increments
+          the final 32 bits of the IV, as in NIST Special Publication
+          800-38D.
 
         A detailed description of these modes is beyond the scope of
         this documentation; refer to a good cryptography book.
@@ -355,7 +362,9 @@ module Cipher : sig
         note that the blocksize is reduced to [n], but encryption
         speed drops by a factor of [blocksize / n], where [blocksize]
         is the block size of the underlying cipher; moreover, [n]
-        must be between [1] and [blocksize] included. *)
+        must be between [1] and [blocksize] included.  For [CTR_N n],
+        [n] must be between [1] and [blocksize] included.
+        [CTR] is equivalent to [CTR_N blocksize]. *)
 
   val aes: ?mode:chaining_mode -> ?pad:Padding.scheme -> ?iv:string ->
              string -> direction -> transform
@@ -376,9 +385,9 @@ module Cipher : sig
         of the cleartext must be an integral number of blocks.
 
         The optional [iv] argument is the initialization vector used
-        in modes CBC, CFB and OFB.  It is ignored in ECB mode.
-        If provided, it must be a string of the same size as the block size
-        (16 bytes).  If omitted, the null initialization vector
+        by the chaining mode.  It is ignored in ECB mode.  If
+        provided, it must be a string of the same size as the block
+        size (16 bytes).  If omitted, the null initialization vector
         (16 zero bytes) is used.
 
         The [aes] function returns a transform that performs encryption
@@ -456,17 +465,18 @@ module Cipher : sig
         of the cleartext must be an integral number of blocks.
 
         The optional [iv] argument is the initialization vector used
-        in modes CBC, CFB and OFB.  It is ignored in ECB mode.
+        by the chaining mode.  It is ignored in ECB mode.
         If provided, it must be a string of the same size as the block size
-        (16 bytes).  If omitted, the null initialization vector
-        (16 zero bytes) is used.
+        (8 bytes).  If omitted, the null initialization vector
+        (8 zero bytes) is used.
 
         The [blowfish] function returns a transform that performs encryption
         or decryption, depending on the direction argument. *)
 end
 
 (** The [Hash] module implements unkeyed cryptographic hashes (SHA-1,
-    SHA-256, RIPEMD-160 and MD5), also known as message digest functions.  
+    SHA-256, SHA-512, SHA-3, RIPEMD-160 and MD5), also known as
+    message digest functions.
     Hash functions used in cryptography are characterized as being
     <I>one-way</I> (given a hash value, it is computationally
     infeasible to find a text that hashes to this value) and
@@ -479,14 +489,24 @@ module Hash : sig
     (** SHA-1 is the Secure Hash Algorithm revision 1.  It is a NIST
         standard, is widely used, and produces 160-bit hashes (20 bytes).
         Recent results suggest that it may not be collision-resistant. *)
-  val sha256: unit -> hash
-    (** SHA-256, another NIST standard, is a variant of SHA-1 that
-        produces 256-bit hashes (32 bytes). *)
+  val sha2: int -> hash
+    (** SHA-2, another NIST standard for cryptographic hashing, produces
+        hashes of 224, 256, 384, or 512 bits (24, 32, 48 or 64 bytes).
+        The parameter is the desired size of the hash, in
+        bits.  It must be one of 224, 256, 384 or 512. *)
   val sha3: int -> hash
     (** SHA-3, the latest NIST standard for cryptographic hashing,
         produces hashes of 224, 256, 384 or 512 bits (24, 32, 48 or 64
         bytes).  The parameter is the desired size of the hash, in
         bits.  It must be one of 224, 256, 384 or 512. *)
+  val sha224: unit -> hash
+    (** SHA-224 is SHA-2 specialized to 224 bit hashes (24 bytes). *)
+  val sha256: unit -> hash
+    (** SHA-256 is SHA-2 specialized to 256 bit hashes (32 bytes). *)
+  val sha384: unit -> hash
+    (** SHA-384 is SHA-2 specialized to 384 bit hashes (48 bytes). *)
+  val sha512: unit -> hash
+    (** SHA-512 is SHA-2 specialized to 512 bit hashes (64 bytes). *)
   val ripemd160: unit -> hash
     (** RIPEMD-160 produces 160-bit hashes (20 bytes). *)
   val md5: unit -> hash
@@ -507,8 +527,8 @@ end
     and if it matches the transmitted MAC, be reasonably certain that
     the text was authentified by someone who possesses the secret key.
 
-    The module [MAC] provides four MAC functions based on the hashes
-    SHA-1, SHA256, RIPEMD160 and MD5, and four MAC functions based on
+    The module [MAC] provides five MAC functions based on the hashes
+    SHA-1, SHA256, SHA512, RIPEMD160 and MD5, and four MAC functions based on
     the block ciphers AES, DES, and Triple-DES. *)
 module MAC: sig
   val hmac_sha1: string -> hash
@@ -522,6 +542,12 @@ module MAC: sig
         256 bits (32 bytes) long.  The [key] argument is the MAC key;
         it can have any length, but a minimal length of 32 bytes is
         recommended. *)
+  val hmac_sha512: string -> hash
+    (** [hmac_sha512 key] returns a MAC based on the HMAC construction
+        (RFC2104) applied to SHA-512.  The returned hash values are
+        512 bits (64 bytes) long.  The [key] argument is the MAC key;
+        it can have any length, but a minimal length of 64 bytes is
+        recommended. *)
   val hmac_ripemd160: string -> hash
     (** [hmac_ripemd160 key] returns a MAC based on the HMAC
         construction (RFC2104) applied to RIPEMD-160.  The returned
@@ -860,6 +886,19 @@ module Block : sig
         the block size of the underlying cipher, included.        
         The returned block cipher has block size [n].
         It is usable both for encryption and decryption. *)
+  class ctr: ?iv: string -> ?inc:int -> block_cipher -> block_cipher
+    (** Add Counter mode to the given block cipher.  Viewing the IV
+        as a [blocksize]-byte integer in big-endian representation,
+        the blocks [IV], [IV+1], [IV+2], ... are encrypted using
+        the given block cipher, and the result is xor-ed with the
+        input blocks to produce the output blocks.  The additions
+        [IV+n] are performed modulo 2 to the [8 * inc] power.
+        In other words, only the low [inc] bytes of the [IV] are
+        subject to incrementation; the high [blocksize - inc] bytes
+        are unaffected.  [inc] defaults to [blocksize].
+        The returned block cipher has the same block size as
+        the underlying block cipher, and is usable both for
+        encryption and decryption. *)
 end
 
 (** The [Stream] module provides classes that implement
